extern crate serde;
extern crate serde_json;
extern crate strfmt;

use serde::{Deserialize, Serialize};

use std::collections::HashMap;
use std::fs;
use std::path::Path;

pub struct Config {
    lang_dir: String,
    lang_ids: Vec<String>,
}

impl Config {
    pub fn new(lang_dir: String, lang_ids: Vec<String>) -> Self {
        Config {
            lang_dir,
            lang_ids,
        }
    }
}

#[derive(Debug, PartialEq)]
pub enum TranslationResult {
    Ok(String),
    KeyNotFound,
    LanguageNotFound,
}

pub struct Uebersetzt {
    config: Config,
    langs: HashMap<String, Language>,
}

impl Uebersetzt {
    pub fn new(config: Config)  -> Self {
        let loaded_langs: Vec<Language> = config.lang_ids.iter()
            .map(|l| load_lang(Path::new(&config.lang_dir), l).unwrap())
            .collect();
        let mut langs = HashMap::<String, Language>::new();
        for lang in loaded_langs {
            langs.insert(lang.id.clone(), lang);
        }
        Uebersetzt {
            config,
            langs,
        }
    }
    pub fn load(&mut self, lang_id: &str) -> Result<(), String> {
        match load_lang(Path::new(&self.config.lang_dir), lang_id) {
            Ok(l) => {
                self.langs.insert(lang_id.to_string(), l);
                Ok(())
            },
            Err(e) => Err(e),
        }
    }
    pub fn insert(&mut self, lang: Language) {
        self.langs.insert(lang.id.clone(), lang);
    }
    pub fn get(&self, lang_id: &str, key: &str) -> TranslationResult {
        match self.langs.get(lang_id) {
            Some(l) => l.translate(key),
            None => TranslationResult::LanguageNotFound,
        }
    }
}

impl TranslationResult {
    pub fn unwrap(self) -> String {
        match self {
            TranslationResult::Ok(t) => t,
            TranslationResult::KeyNotFound =>
                panic!("Tried unwrapping translation which could not be found"),
            TranslationResult::LanguageNotFound =>
                panic!("Tried unwrapping translation for which the language could not be found"),
        }
    }
}

#[derive(Serialize, Deserialize)]
pub struct Language {
    name: String,
    id: String,
    data: HashMap<String, String>,
}

impl Language {
    pub fn new(name: String, id: String) -> Self {
        Language {
            name,
            id,
            data: HashMap::new(),
        }
    }

    pub fn name(&self) -> String {
        self.name.clone()
    }

    pub fn id(&self) -> String {
        self.id.clone()
    }

    pub fn insert(&mut self, key: String, value: String) {
        self.data.insert(key, value);
    }

    pub fn translate(&self, key: &str) -> TranslationResult {
        match self.data.get(key) {
            Some(t) => TranslationResult::Ok(t.clone()),
            None => TranslationResult::KeyNotFound,
        }
    }
}

pub fn load_lang(lang_dir: &Path, id: &str) -> Result<Language, String> {
    let file = lang_dir.join(Path::new(&format!("{}.json", id)));
    let file = file.as_path();
    let json = fs::read_to_string(file).unwrap();
    let lang = lang_from_json(&json).unwrap();
    if &lang.id == id {
        Ok(lang)
    } else {
        Err(format!("Id in file [{}] does not equal id in file name [{}]", lang.id, id))
    }
}

pub fn lang_from_json(json: &str) -> Result<Language, serde_json::Error> {
    serde_json::from_str(json)
}

#[macro_export]
macro_rules! i18n {
    ($uebersetzt: expr, $lang: expr, $key: expr, $($x: expr),*) => {{
        let mut map: HashMap<String, String> = HashMap::new();
        $(let (k, v) = $x; map.insert(k.to_string(), v.to_string());)*
        let translated = $uebersetzt.get($lang, $key).unwrap();
        let translated = strfmt(&translated, &map).unwrap();
        translated
    }}
}